
import urwid
from subprocess import run, PIPE


class CreateNamespaceMenu:

    def __init__(self, return_func):

        self.create_namespace_edit_box = urwid.Edit(('editcp', u"Namespace to add: "), "set name", align='left', )
        self.create_namespace_result = urwid.Text(u"")

        blank = urwid.Divider()
        listbox_content = [
            blank,
            urwid.WidgetWrap(urwid.Divider("=", 1)),
            urwid.Padding(urwid.Text(u"Create Namespace"), left=2, right=2, min_width=20),
            urwid.WidgetWrap(urwid.Divider("*", 0, 1)),
            blank,

            blank,
            urwid.Padding(urwid.AttrWrap(self.create_namespace_edit_box, 'editbx', 'editfc'), left=2, right=2),
            blank,
            urwid.Padding(
                urwid.GridFlow(
                    [urwid.AttrWrap(urwid.Button("Cancel", on_press=self.on_cancel), 'buttn', 'buttnf'),
                     blank,
                     urwid.AttrWrap(urwid.Button("Create", on_press=self.on_create_namespace), 'buttn', 'buttnf')
                     ],
                    20, 1, 8, 'left'),
                left=2, right=2, min_width=20, align='left'),

            blank,
            urwid.Padding(self.create_namespace_result, left=2, right=2, min_width=20),
        ]

        listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))

        self.return_func = return_func
        self.main_window = urwid.LineBox(urwid.AttrWrap(listbox, 'body'))

    def on_cancel(self, w):
        self.return_func()

    def on_create_namespace(self, w):
        """
        Execute once 'Create' button pressed,
        init the 'create_namespace' function,
        and update 'create_namespace_result' urwid.TEXT object

        :return:
        """

        # Execute the 'create_namespace' function with relevant values, and get the 'value' return from function
        create_namespace_output = create_namespace(self.create_namespace_edit_box.edit_text)
        if create_namespace_output['status'] != 0:
            self.create_namespace_result.set_text(("errors", create_namespace_output['value']))
        else:
            return self.return_func(refresh_namespaces=True, returned_result=create_namespace_output['value'])


def create_namespace(namespace_name):
    """
    Execute 'kubectl create namespace' command on input values

    :param namespace_name: name of namespace as string
    :return: return code and value from execution command as dict
    """
    # execute and get CompletedProcess object
    completed_process_object = run(["kubectl", "create", "namespace", namespace_name], stdout=PIPE, stderr=PIPE)
    if completed_process_object.returncode == 0:
        value = completed_process_object.stdout.decode('utf-8')
    else:
        value = completed_process_object.stderr.decode('utf-8')
    return {'status': completed_process_object.returncode, 'value': value}




from subprocess import run, PIPE


def get_cluster_info():
    """
    Get cluster information

    :return: return value from command execution as string
    """
    cluster_info_completed_process = run(["kubectl", "cluster-info"], stdout=PIPE, stderr=PIPE)

    if cluster_info_completed_process.returncode == 0:
        return cluster_info_completed_process.stdout.decode('utf-8').strip()
    else:
        return cluster_info_completed_process.stderr.decode('utf-8').strip()


def create_registry_secret(secret_name, docker_registry, docker_username, docker_password, namespace):
    """
    Create a new docker-registry secret

    :param secret_name: String
    :param docker_registry: String
    :param docker_username: String
    :param docker_password: String
    :param namespace: String
    :return: return code and value from command execution as dictionary
    """
    add_registry_completed = run(["kubectl", "create", "secret", "docker-registry", secret_name,
                                             "--docker-server=%s" % docker_registry,
                                             "--docker-username=%s" % docker_username,
                                             "--docker-password=%s" % docker_password,
                                             "--namespace=%s" % namespace],
                                 stdout=PIPE,
                                 stderr=PIPE)
    if add_registry_completed.returncode == 0:
        value = add_registry_completed.stdout.decode('utf-8').strip()
    else:
        value = add_registry_completed.stderr.decode('utf-8').strip()

    return {'returncode': add_registry_completed.returncode, 'value': value}


def identify_cluster_namespaces():
    """
    Execute 'kubectl get namespaces' command and return dict with status and value as decoded (utf-8) string

    'kubectl get namespaces' command return example:
    NAME              STATUS    AGE
    default           Active    6d
    docker            Active    6d
    ingress-traefik   Active    5d4h
    kube-public       Active    6d
    kube-system       Active    6d

    :return: return code and value from execution command decoded string
    """
    # Execute 'kubectl get namespaces' command, returned as CompletedProcess
    namespaces_completed_process = run(["kubectl", "get", "namespaces"], stdout=PIPE, stderr=PIPE)

    status = namespaces_completed_process.returncode
    # If return code is not 0
    if status:
        return {'status': status, 'value': namespaces_completed_process.stderr.decode('utf-8').strip()}
    else:
        return {'status': status, 'value': namespaces_completed_process.stdout.decode('utf-8').strip()}


def parse_cluster_namespaces_output(get_namespaces_output):
    """
    Function will perform a manipulation on a string output from the 'kubectl get namespaces' command

    Returns an array of dicts with installed repos names and urls as strings
    as [{'name': 'some_namespace', 'status': 'active', 'age': '5h20m'}]

    by validating the first line, splitting by the tab delimiter,
    and checking that the first (0) value is 'NAME' second (1) value is 'STATUS' and third (2) is 'AGE'
    an exception will be raised if the structure was change by kubectl developers

    :param get_namespaces_output: 'kubectl get namespaces' output as String
    :return:
    """
    available_namespaces = []

    # split get_namespaces_output by 'new line'
    get_namespaces_stdout = get_namespaces_output.split("\n")
    # Perform validation on stdout of first (0) line
    first_line_stdout = get_namespaces_stdout[0].split()
    if first_line_stdout[0].strip() != 'NAME' or \
            first_line_stdout[1].strip() != 'STATUS' or \
            first_line_stdout[2].strip() != 'AGE':
        raise Exception("'kubectl get namespaces' command output changed, "
                        "code change is needed to resolve this issue, "
                        "contact the developer.")

    # for every line in existing namespaces, excluding the headers line (NAME, STATUS and AGE)
    for line in get_namespaces_stdout[1:]:
        # each stdout 'kubectl get namespaces' line composed by tabs delimiter, split it
        namespace_details = line.split()

        temp_dictionary = {}
        if namespace_details[0] != "":
            # Add current line repo values to dict
            temp_dictionary.update({'name': namespace_details[0].strip()})
            temp_dictionary.update({'status': namespace_details[1].strip()})
            temp_dictionary.update({'age': namespace_details[2].strip()})
            # Update final array with the temp array of dicts of current repo
            available_namespaces.append(temp_dictionary)

    return available_namespaces




import re


def remove_ansi_color_from_string(input_string):
    ansi_escape = re.compile(r'(\x9B|\x1B\[)[[0-?]*[ -/]*[@-~]')
    return ansi_escape.sub('', input_string)


def is_valid_charts_yaml(content):
    """
    Check if 'content' contains mandatory keys

    :param content: parsed YAML file as list of dictionary of key values
    :return: True if dict contains mandatory values, else False
    """

    # Iterate on each list cell
    for chart_details in content:
        # If one of the keys is missing or, is None
        if not all(chart_details.get(x) is not None
                   and x in chart_details
                   for x in ['chart_name', 'helm_repo_name', 'name_space', 'values_file', 'private_image']):
            return False
        # If one of the keys is not a string
        if not all(type(chart_details.get(x)) is str
                   for x in ['chart_name', 'helm_repo_name', 'name_space', 'values_file']):
            return False
        # If one of the keys is not a boolean
        if not all(type(chart_details.get(x)) is bool
                   for x in ['private_image']):
            return False
        if not all(type(chart_details.get(x)) is list
                   for x in ['extra_executes']):
            return False
    return True


main_palette = [
    ('body', 'black', 'light gray', 'standout'),
    ('reverse', 'light gray', 'black'),
    ('header', 'white', 'dark red', 'bold'),
    ('footer', 'white', 'dark green', 'bold'),
    ('important', 'dark blue', 'light gray', ('standout', 'underline')),
    ('editfc', 'white', 'dark blue', 'bold'),
    ('editbx', 'light gray', 'dark blue'),
    ('editcp', 'black', 'light gray', 'standout'),
    ('bright', 'dark gray', 'light gray', ('bold', 'standout')),
    ('buttn', 'black', 'light blue'),
    ('buttnf', 'white', 'dark blue', 'bold'),
    ('errors', 'black', 'dark red'),
]




import os
from subprocess import run, PIPE


values_dir_path = os.path.dirname(os.path.realpath(__file__)) + '/values_files/'

supported_helm_deployments = [
    {'chart_name': 'airflow',
     'helm_repo_name': 'stable/airflow',
     'name_space': 'airflow',
     'values_file': 'airflow.values.local.yml',
     'private_image': False,
     'extra_executes': []
     },

    {'chart_name': 'ingress-traefik',
     'helm_repo_name': 'stable/traefik',
     'name_space': 'ingress-traefik',
     'values_file': 'ingress-traefik.values.local.yml',
     'private_image': False,
     'extra_executes': []},

    {'chart_name': 'kubernetes-dashboard',
     'helm_repo_name': 'stable/kubernetes-dashboard',
     'name_space': 'kube-system',
     'values_file': 'kubernetes-dashboard.values.local.yml',
     'private_image': False,
     'extra_executes': []},

    {'chart_name': 'jenkins',
     'helm_repo_name': 'stable/jenkins',
     'name_space': 'jenkins',
     'values_file': 'jenkins.values.local.yml',
     'private_image': False,
     'extra_executes': [
         'kubectl apply -f jenkins.configmap.yml --namespace jenkins',
     ]},

    {'chart_name': 'openfaas',
     'helm_repo_name': 'openfaas/openfaas',
     'name_space': 'openfaas',
     'values_file': 'openfaas.values.local.yml',
     'private_image': False,
     'extra_executes': [
         '',
     ]
     },

    {'chart_name': 'mysql',
     'helm_repo_name': 'stable/mysql',
     'name_space': 'mysql',
     'values_file': 'mysql.values.local.yml',
     'private_image': False,
     'extra_executes': []},

    {'chart_name': 'rabbitmq',
     'helm_repo_name': 'stable/rabbitmq',
     'name_space': 'rabbitmq',
     'values_file': 'rabbitmq.values.local.yml',
     'private_image': False,
     'extra_executes': []},

    {'chart_name': 'redis',
     'helm_repo_name': 'stable/redis',
     'name_space': 'redis',
     'values_file': 'redis.values.local.yml',
     'private_image': False,
     'extra_executes': []},

    {'chart_name': 'nalkinscloud-frontend',
     'helm_repo_name': 'nalkinscloud/nalkinscloud',
     'name_space': 'nalkinscloud-frontend',
     'values_file': 'nalkinscloud.frontend.values.local.yml',
     'private_image': True,
     'extra_executes': []},

    {'chart_name': 'nalkinscloud-api',
     'helm_repo_name': 'nalkinscloud/nalkinscloud',
     'name_space': 'nalkinscloud-api',
     'values_file': 'nalkinscloud.api.values.local.yml',
     'private_image': True,
     'extra_executes': []},
]


def identify_installed_helm_repos():
    """
    Execute 'helm repo list' command and return dict with status and value as decoded (utf-8) string

    'helm repo list' command return example:
    NAME        	URL
    stable      	https://kubernetes-charts.storage.googleapis.com
    local       	http://127.0.0.1:8879/charts
    nalkinscloud	https://arielevs.github.io/Kubernetes-Helm-Charts/

    :return: return code and value from execution command decoded string
    """
    # Execute 'helm list' command, returned as CompletedProcess
    installed_repos_completed_process = run(["helm", "repo", "list"], stdout=PIPE, stderr=PIPE)

    status = installed_repos_completed_process.returncode
    # If return code is not 0
    if status:
        return {'status': status, 'value': installed_repos_completed_process.stderr.decode('utf-8').strip()}
    else:
        return {'status': status, 'value': installed_repos_completed_process.stdout.decode('utf-8').strip()}


def parse_helm_repo_list_output(helm_repo_list_output):
    """
    Function will perform a manipulation on a string output from the 'helm repo list' command

    Returns an array of dicts with installed repos names and urls as strings
    as [{'repo_name': 'some_repo_name', 'repo_url': 'some_repo_url'}]

    by validating the first line, splitting by the tab delimiter,
    and checking that the first (0) value is 'NAME' and second (1) value is 'URL'
    an exception will be raised if the structure was change by HELM developers

    :param helm_repo_list_output: 'helm repo list' output as String
    :return:
    """
    installed_repos = []

    # split helm_repo_list_output by 'new line'
    installed_repos_stdout = helm_repo_list_output.split("\n")

    # Perform validation on stdout of first (0) line
    first_line_stdout = installed_repos_stdout[0].split("\t")
    if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[1].strip() != 'URL':
        raise Exception("'helm repo list' command output changed, "
                        "code change is needed to resolve this issue, "
                        "contact the developer.")

    # for every line in installed repos, excluding the headers line (NAME and URL)
    for line in installed_repos_stdout[1:]:
        # each stdout 'helm repo list' line composed by tabs delimiter, split it
        repo_details = line.split("\t")

        temp_dictionary = {}
        if repo_details[0] != "":
            # Add current line repo values to dict
            temp_dictionary.update({'repo_name': repo_details[0].strip()})
            temp_dictionary.update({'repo_url': repo_details[1].strip()})
            # Update final array with the temp array of dicts of current repo
            installed_repos.append(temp_dictionary)

    return installed_repos


def identify_installed_helm_charts():
    """
    Execute 'helm list' command and return dict with status and value as decoded (utf-8) string

    'helm list' command return example:
    NAME                	REVISION	UPDATED                 	STATUS  	CHART                     	APP VERSION     NAMESPACE
    ingress-traefik     	2       	Thu Dec 27 19:45:01 2018	DEPLOYED	traefik-1.56.0            	1.7.6           ingress-traefik
    kubernetes-dashboard	11      	Sun Sep 16 11:21:24 2018	DEPLOYED	kubernetes-dashboard-0.7.3	1.10.1          kube-system

    :return: array of dicts with helm installations
    """
    # Execute 'helm list' command, returned as CompletedProcess
    installed_helm_completed_process = run(["helm", "list"], stdout=PIPE, stderr=PIPE)

    status = installed_helm_completed_process.returncode
    # If return code is not 0
    if status:
        return {'status': status, 'value': installed_helm_completed_process.stderr.decode('utf-8').strip()}
    else:
        return {'status': status, 'value': installed_helm_completed_process.stdout.decode('utf-8').strip()}


def parse_helm_list_output(helm_list_output):
    """
    Function will perform a manipulation on a string output from the 'helm list' command

    Returns an array of dicts with installed chart names and namespaces as strings
    as [{'chart_name': 'some_chart_name', 'name_space': 'some_name_space'}]

    by validating the first line, splitting by the tab delimiter,
    and checking that the first (0) value is 'NAME' and seventh (6) value is 'NAMESPACE'
    an exception will be raised if the structure was change by HELM developers

    :param helm_list_output: 'helm list' output as String
    :return: list of dicts
    """
    installed_charts = []

    # split helm_list_output by 'new line'
    installed_helm_stdout = helm_list_output.split("\n")

    # Perform validation on stdout of first (0) line
    first_line_stdout = installed_helm_stdout[0].split("\t")
    if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[6].strip() != 'NAMESPACE':
        raise Exception("'helm list' command output changed, probably due to helm version update, "
                        "code change is needed to resolve this issue, "
                        "contact the developer.")

    # for every line in installed charts, excluding the headers line (Name, Revision, Updated etc...)
    for line in installed_helm_stdout[1:]:
        # each stdout 'helm list' line composed by tabs delimiter, split it
        chart_details = line.split("\t")

        temp_dictionary = {}
        if chart_details[0] != "":
            # Add current line chart values to dict
            temp_dictionary.update({'chart_name': chart_details[0].strip()})
            temp_dictionary.update({'name_space': chart_details[6].strip()})
            # Update final array with the temp array of dicts of current helm deployment
            installed_charts.append(temp_dictionary)

    return installed_charts


def identify_charts_in_search(repository_name):
    """
    Execute 'helm repo update', and 'helm search repository_name' command on input values

    :param repository_name: name of repository as strings
    :return: return code and value from execution command decoded string
    """

    # Update helm repo before installation
    # --strict will fail on update warnings
    # TODO repo update will fail even if one of the repositories returned error,
    # feature requested at https://github.com/helm/helm/issues/5127
    # Add '--strict' and '--repo-name' if helm api is updated
    completed_process_object = run(["helm", "repo", "update"], stdout=PIPE, stderr=PIPE)
    if completed_process_object.returncode != 0:
        return {'status': completed_process_object.returncode,
                'value': completed_process_object.stderr.decode('utf-8').strip()}

    # execute and get CompletedProcess object
    completed_process_object = run(["helm", "search", repository_name], stdout=PIPE, stderr=PIPE)

    # If return code is not 0
    if completed_process_object.returncode:
        return {'status': completed_process_object.returncode,
                'value': completed_process_object.stderr.decode('utf-8').strip()}
    else:
        return {'status': completed_process_object.returncode,
                'value': completed_process_object.stdout.decode('utf-8').strip()}


def parse_helm_search_output(helm_search_output):
    # split stdout of helm_search_output by 'new line'
    helm_search_rows_list = helm_search_output.split("\n")

    # Perform validation on stdout of first (0) line
    first_line_stdout = helm_search_rows_list[0].split("\t")
    if first_line_stdout[0].strip() != 'NAME' or first_line_stdout[3].strip() != 'DESCRIPTION':
        raise Exception("'helm search' command output changed, probably due to helm version update, "
                        "code change is needed to resolve this issue, "
                        "contact the developer.")

    charts_list = []
    # for every line in helm search repo output, excluding the headers line (NAME, CHART VERSION etc)
    for line in helm_search_rows_list[1:]:
        # each stdout 'helm search' line composed by tabs delimiter, split it
        chart_details = line.split("\t")

        temp_dictionary = {}
        if chart_details[0] != "":
            # Add current line chart values to dict
            temp_dictionary.update({'chart_name': chart_details[0].strip()})
            temp_dictionary.update({'chart_version': chart_details[1].strip()})
            temp_dictionary.update({'app_version': chart_details[2].strip()})
            temp_dictionary.update({'description': chart_details[3].strip()})
            # Update final array with the temp array of dicts of current charts
            charts_list.append(temp_dictionary)

    return charts_list
